@@ -48,8 +48,12 @@ __all__ = (
     'SecurityLevel',
     'Status',
     'User',
+    'Group',
     'CustomFieldOption',
-    'RemoteLink'
+    'RemoteLink',
+    'Customer',
+    'ServiceDesk',
+    'RequestType',
 )
 
 logging.getLogger('jira').addHandler(NullHandler())
@@ -201,17 +205,17 @@ class Resource(object):
         options.update({'path': path})
         return self._base_url.format(**options)
 
-    def update(self, fields=None, async=None, jira=None, notify=True, **kwargs):
+    def update(self, fields=None, async_=None, jira=None, notify=True, **kwargs):
         """Update this resource on the server.
 
         Keyword arguments are marshalled into a dict before being sent. If this
         resource doesn't support ``PUT``, a :py:exc:`.JIRAError` will be raised; subclasses that specialize this method
         will only raise errors in case of user error.
 
-        :param async: if true the request will be added to the queue so it can be executed later using async_run()
+        :param async_: if true the request will be added to the queue so it can be executed later using async_run()
         """
-        if async is None:
-            async = self._options['async']
+        if async_ is None:
+            async_ = self._options['async']
 
         data = {}
         if fields is not None:
@@ -279,8 +283,7 @@ class Resource(object):
                 #    logging.warning("autofix: setting assignee to '%s' and retrying the update." % self._options['autofix'])
                 #    data['fields']['assignee'] = {'name': self._options['autofix']}
             # EXPERIMENTAL --->
-            # import grequests
-            if async:
+            if async_:
                 if not hasattr(self._session, '_async_jobs'):
                     self._session._async_jobs = set()
                 self._session._async_jobs.add(threaded_requests.put(
@@ -328,7 +331,7 @@ class Resource(object):
 
     def _default_headers(self, user_headers):
         # result = dict(user_headers)
-        # esult['accept'] = 'application/json'
+        # result['accept'] = 'application/json'
         return CaseInsensitiveDict(self._options['headers'].items() + user_headers.items())
 
 
@@ -421,15 +424,15 @@ class Issue(Resource):
         Resource.__init__(self, 'issue/{0}', options, session)
 
         self.fields = None
-        """ :type : Issue._IssueFields """
+        """ :type: :class:`~Issue._IssueFields` """
         self.id = None
-        """ :type : int """
+        """ :type: int """
         self.key = None
-        """ :type : str """
+        """ :type: str """
         if raw:
             self._parse_raw(raw)
 
-    def update(self, fields=None, update=None, async=None, jira=None, notify=True, **fieldargs):
+    def update(self, fields=None, update=None, async_=None, jira=None, notify=True, **fieldargs):
         """Update this issue on the server.
 
         Each keyword argument (other than the predefined ones) is treated as a field name and the argument's value
@@ -441,10 +444,12 @@ class Issue(Resource):
         are available here: https://developer.atlassian.com/display/JIRADEV/JIRA+REST+API+Example+-+Edit+issues
 
         :param fields: a dict containing field names and the values to use
-
         :param update: a dict containing update operations to apply
+        :param fieldargs: keyword arguments will generally be merged into fields, except lists,
+                          which will be merged into updates
 
-        keyword arguments will generally be merged into fields, except lists, which will be merged into updates
+        :type fields: dict
+        :type update: dict
         """
         data = {}
         if fields is not None:
@@ -477,12 +482,17 @@ class Issue(Resource):
             else:
                 fields_dict[field] = value
 
-        super(Issue, self).update(async=async, jira=jira, notify=notify, fields=data)
+        super(Issue, self).update(async_=async_, jira=jira, notify=notify, fields=data)
 
     def add_field_value(self, field, value):
         """Add a value to a field that supports multiple values, without resetting the existing values.
 
         This should work with: labels, multiple checkbox lists, multiple select
+
+        :param field: The field name
+        :param value: The field's value
+
+        :type field: str
         """
         super(Issue, self).update(fields={"update": {field: [{"add": value}]}})
 
@@ -490,6 +500,8 @@ class Issue(Resource):
         """Delete this issue from the server.
 
         :param deleteSubtasks: if the issue has subtasks, this argument must be set to true for the call to succeed.
+
+        :type deleteSubtasks: bool
         """
         super(Issue, self).delete(params={'deleteSubtasks': deleteSubtasks})
 
@@ -497,6 +509,8 @@ class Issue(Resource):
         """Get the URL of the issue, the browsable one not the REST one.
 
         :return: URL of the issue
+
+        :rtype: str
         """
         return "%s/browse/%s" % (self._options['server'], self.key)
 
@@ -513,7 +527,7 @@ class Comment(Resource):
         if raw:
             self._parse_raw(raw)
 
-    def update(self, fields=None, async=None, jira=None, body='', visibility=None):
+    def update(self, fields=None, async_=None, jira=None, body='', visibility=None):
         data = {}
         if body:
             data['body'] = body
@@ -672,6 +686,7 @@ class Role(Resource):
         :param groups: a group or groups to add to the role
         :type groups: string, list or tuple
         """
+
         if users is not None and isinstance(users, string_types):
             users = (users,)
         if groups is not None and isinstance(groups, string_types):
@@ -685,6 +700,26 @@ class Role(Resource):
 
         super(Role, self).update(**data)
 
+    def add_user(self, users=None, groups=None):
+        """Add the specified users or groups to this project role.
+
+        One of ``users`` or ``groups`` must be specified.
+
+        :param users: a user or users to add to the role
+        :type users: string, list or tuple
+        :param groups: a group or groups to add to the role
+        :type groups: string, list or tuple
+        """
+
+        if users is not None and isinstance(users, string_types):
+            users = (users,)
+        if groups is not None and isinstance(groups, string_types):
+            groups = (groups,)
+
+        data = {
+            'user': users}
+        self._session.post(self.self, data=json.dumps(data))
+
 
 class Resolution(Resource):
     """A resolution for an issue."""
@@ -722,7 +757,7 @@ class User(Resource):
             self._parse_raw(raw)
 
     def __hash__(self):
-        """Hash carculation."""
+        """Hash calculation."""
         return hash(str(self.name))
 
     def __eq__(self, other):
@@ -730,6 +765,23 @@ class User(Resource):
         return str(self.name) == str(other.name)
 
 
+class Group(Resource):
+    """A JIRA user group."""
+
+    def __init__(self, options, session, raw=None):
+        Resource.__init__(self, 'group?groupname={0}', options, session)
+        if raw:
+            self._parse_raw(raw)
+
+    def __hash__(self):
+        """Hash calculation."""
+        return hash(str(self.name))
+
+    def __eq__(self, other):
+        """Equality by name."""
+        return str(self.name) == str(other.name)
+
+
 class Version(Resource):
     """A version of a project."""
 
@@ -749,6 +801,7 @@ class Version(Resource):
         :param moveAffectedIssuesTo: in issues for which this version is an
             affected version, add this argument version to the affected version list
         """
+
         params = {}
         if moveFixIssuesTo is not None:
             params['moveFixIssuesTo'] = moveFixIssuesTo
@@ -825,6 +878,34 @@ class Board(GreenHopperResource):
         Resource.delete(self, params)
 
 
+# Service Desk
+
+class Customer(Resource):
+    """A Service Desk customer."""
+
+    def __init__(self, options, session, raw=None):
+        Resource.__init__(self, 'customer', options, session, '{server}/rest/servicedeskapi/{path}')
+        if raw:
+            self._parse_raw(raw)
+
+
+class ServiceDesk(Resource):
+    """A Service Desk."""
+
+    def __init__(self, options, session, raw=None):
+        Resource.__init__(self, 'servicedesk/{0}', options, session, '{server}/rest/servicedeskapi/{path}')
+        if raw:
+            self._parse_raw(raw)
+
+
+class RequestType(Resource):
+    """A Service Desk Request Type."""
+
+    def __init__(self, options, session, raw=None):
+        Resource.__init__(self, 'servicedesk/{0}/requesttype', options, session, '{server}/rest/servicedeskapi/{path}')
+        if raw:
+            self._parse_raw(raw)
+
 # Utilities
 
 
@@ -867,6 +948,7 @@ def dict2resource(raw, top=None, options=None, session=None):
             setattr(top, i, j)
     return top
 
+
 resource_class_map = {
     # JIRA specific resources
     r'attachment/[^/]+$': Attachment,
@@ -889,16 +971,26 @@ resource_class_map = {
     r'securitylevel/[^/]+$': SecurityLevel,
     r'status/[^/]+$': Status,
     r'user\?username.+$': User,
+    r'group\?groupname.+$': Group,
     r'version/[^/]+$': Version,
     # GreenHopper specific resources
     r'sprints/[^/]+$': Sprint,
     r'views/[^/]+$': Board}
 
 
+class UnknownResource(Resource):
+    """A Resource from JIRA that is not (yet) supported."""
+
+    def __init__(self, options, session, raw=None):
+        Resource.__init__(self, 'unknown{0}', options, session)
+        if raw:
+            self._parse_raw(raw)
+
+
 def cls_for_resource(resource_literal):
     for resource in resource_class_map:
         if re.search(resource, resource_literal):
             return resource_class_map[resource]
     else:
-        # Generic Resource without specialized update/delete behavior
-        return Resource
+        # Generic Resource cannot directly be used b/c of different constructor signature
+        return UnknownResource
