@@ -30,7 +30,7 @@ from tenacity import stop_after_attempt
 _non_parallel = True
 if platform.python_version() < '3':
     _non_parallel = False
-
+    import mock
     try:
         import unittest2 as unittest
     except ImportError:
@@ -43,6 +43,11 @@ if platform.python_version() < '3':
         import unittest2 as unittest
 else:
     import unittest
+    try:
+        from unittest import mock
+    except ImportError:
+        import mock
+
 
 cmd_folder = os.path.abspath(os.path.join(os.path.split(inspect.getfile(
     inspect.currentframe()))[0], ".."))
@@ -51,7 +56,7 @@ if cmd_folder not in sys.path:
 
 import jira  # noqa
 from jira import Role, Issue, JIRA, JIRAError, Project  # noqa
-from jira.resources import Resource, cls_for_resource   # noqa
+from jira.resources import Resource, cls_for_resource, Group, UnknownResource  # noqa
 
 TEST_ROOT = os.path.dirname(__file__)
 TEST_ICON_PATH = os.path.join(TEST_ROOT, 'icon.png')
@@ -82,7 +87,7 @@ def rndstr():
 
 
 def rndpassword():
-    # generates a password of lengh 14
+    # generates a password of length 14
     s = ''.join(random.sample(string.ascii_uppercase, 5)) + \
         ''.join(random.sample(string.ascii_lowercase, 5)) + \
         ''.join(random.sample(string.digits, 2)) + \
@@ -97,10 +102,9 @@ def hashify(some_string, max_len=8):
 def get_unique_project_name():
     jid = ""
     user = re.sub("[^A-Z_]", "", getpass.getuser().upper())
-
     if user == 'TRAVIS' and 'TRAVIS_JOB_NUMBER' in os.environ:
-        # please note that user underline (_) is not suppored by
-        # jira even if is documented as supported.
+        # please note that user underline (_) is not supported by
+        # JIRA even if it is documented as supported.
         jid = 'T' + hashify(user + os.environ['TRAVIS_JOB_NUMBER'])
     else:
         identifier = user + \
@@ -110,18 +114,6 @@ def get_unique_project_name():
     return jid
 
 
-class Singleton(type):
-
-    def __init__(cls, name, bases, dict):
-        super(Singleton, cls).__init__(name, bases, dict)
-        cls.instance = None
-
-    def __call__(cls, *args, **kw):
-        if cls.instance is None:
-            cls.instance = super(Singleton, cls).__call__(*args, **kw)
-        return cls.instance
-
-
 class JiraTestManager(object):
     """Used to instantiate and populate the JIRA instance with data used by the unit tests.
 
@@ -131,19 +123,6 @@ class JiraTestManager(object):
         max_retries (int): number of retries to perform for recoverable HTTP errors.
     """
 
-    # __metaclass__ = Singleton
-
-    # __instance = None
-    #
-    # Singleton implementation
-    # def __new__(cls, *args, **kwargs):
-    #     if not cls.__instance:
-    #         cls.__instance = super(JiraTestManager, cls).__new__(
-    #                             cls, *args, **kwargs)
-    #     return cls.__instance
-
-    #  Implementing some kind of Singleton, to prevent test initialization
-    # http://stackoverflow.com/questions/31875/is-there-a-simple-elegant-way-to-define-singletons-in-python/33201#33201
     __shared_state = {}
 
     @retry(stop=stop_after_attempt(2))
@@ -201,7 +180,6 @@ class JiraTestManager(object):
                         self.jira_admin = JIRA(self.CI_JIRA_URL, validate=True,
                                                logging=False, max_retries=self.max_retries)
                 if self.jira_admin.current_user() != self.CI_JIRA_ADMIN:
-                    # self.jira_admin.
                     self.initialized = 1
                     sys.exit(3)
 
@@ -305,21 +283,20 @@ class JiraTestManager(object):
 
                 try:
                     self.jira_admin.create_project(self.project_a,
-                                                   self.project_a_name)
+                                                   self.project_a_name,
+                                                   template_name='Scrum software development')
                 except Exception:
                     # we care only for the project to exist
                     pass
                 self.project_a_id = self.jira_admin.project(self.project_a).id
-                # except Exception as e:
-                #    logging.warning("Got %s" % e)
-                # try:
-                # assert self.jira_admin.create_project(self.project_b,
-                # self.project_b_name) is  True, "Failed to create %s" %
-                # self.project_b
+                self.jira_admin.create_project(self.project_b,
+                                               self.project_b_name,
+                                               template_name='Scrum software development')
 
                 try:
                     self.jira_admin.create_project(self.project_b,
-                                                   self.project_b_name)
+                                                   self.project_b_name,
+                                                   template_name='Scrum software development')
                 except Exception:
                     # we care only for the project to exist
                     pass
@@ -440,7 +417,9 @@ class ResourceTests(unittest.TestCase):
         self.assertEqual(cls_for_resource('http://imaginary-jira.com/rest/\
                 api/latest/project/IMG/role/10002'), Role)
         self.assertEqual(cls_for_resource('http://customized-jira.com/rest/\
-                plugin-resource/4.5/json/getMyObject'), Resource)
+                plugin-resource/4.5/json/getMyObject'), UnknownResource)
+        self.assertEqual(cls_for_resource('http://customized-jira.com/rest/\
+                group?groupname=bla'), Group)
 
 
 @flaky
@@ -464,7 +443,7 @@ class ApplicationPropertiesTests(unittest.TestCase):
     def test_set_application_property(self):
         prop = 'jira.lf.favicon.hires.url'
         valid_value = '/jira-favicon-hires.png'
-        invalid_value = '/Tjira-favicon-hires.png'
+        invalid_value = '/invalid-jira-favicon-hires.png'
 
         self.jira.set_application_property(prop, invalid_value)
         self.assertEqual(self.jira.application_properties(key=prop)['value'],
@@ -494,7 +473,6 @@ class AttachmentTests(unittest.TestCase):
         self.assertTrue(meta['enabled'])
         self.assertEqual(meta['uploadLimit'], 10485760)
 
-    @unittest.skip("TBD: investigate failure")
     def test_1_add_remove_attachment(self):
         issue = self.jira.issue(self.issue_1)
         attachment = self.jira.add_attachment(issue,
@@ -506,7 +484,8 @@ class AttachmentTests(unittest.TestCase):
             new_attachment.filename, 'new test attachment', msg=msg)
         self.assertEqual(
             new_attachment.size, os.path.getsize(TEST_ATTACH_PATH), msg=msg)
-        assert attachment.delete() is None
+        # JIRA returns a HTTP 204 upon successful deletion
+        self.assertEqual(attachment.delete().status_code, 204)
 
 
 @flaky
@@ -575,6 +554,13 @@ class ComponentTests(unittest.TestCase):
         component.delete()
         self.assertRaises(JIRAError, self.jira.component, myid)
 
+    def test_delete_component_by_id(self):
+        component = self.jira.create_component('To be deleted',
+                                               self.project_b, description='not long for this world')
+        myid = component.id
+        self.jira.delete_component(myid)
+        self.assertRaises(JIRAError, self.jira.component, myid)
+
 
 @flaky
 class CustomFieldOptionTests(unittest.TestCase):
@@ -722,7 +708,7 @@ class IssueTests(unittest.TestCase):
         self.assertTrue(issue1 == issues[0])
         self.assertFalse(issue2 == issues[0])
 
-    def test_issue_expandos(self):
+    def test_issue_expand(self):
         issue = self.jira.issue(self.issue_1, expand='editmeta,schema')
         self.assertTrue(hasattr(issue, 'editmeta'))
         self.assertTrue(hasattr(issue, 'schema'))
@@ -732,10 +718,10 @@ class IssueTests(unittest.TestCase):
     @not_on_custom_jira_instance
     def test_create_issue_with_fieldargs(self):
         issue = self.jira.create_issue(project=self.project_b,
-                                       summary='Test issue created', description='blahery',
+                                       summary='Test issue created', description='foo description',
                                        issuetype={'name': 'Bug'})  # customfield_10022='XSS'
         self.assertEqual(issue.fields.summary, 'Test issue created')
-        self.assertEqual(issue.fields.description, 'blahery')
+        self.assertEqual(issue.fields.description, 'foo description')
         self.assertEqual(issue.fields.issuetype.name, 'Bug')
         self.assertEqual(issue.fields.project.key, self.project_b)
         # self.assertEqual(issue.fields.customfield_10022, 'XSS')
@@ -768,7 +754,7 @@ class IssueTests(unittest.TestCase):
         issue = self.jira.create_issue(prefetch=False,
                                        project=self.project_b,
                                        summary='Test issue created',
-                                       description='blahery', issuetype={'name': 'Bug'}
+                                       description='some details', issuetype={'name': 'Bug'}
                                        )  # customfield_10022='XSS'
 
         assert hasattr(issue, 'self')
@@ -787,8 +773,7 @@ class IssueTests(unittest.TestCase):
                 'name': 'Bug'},
             # 'customfield_10022': 'XSS',
             'priority': {
-                'name': 'Major'}},
-            {
+                'name': 'Major'}}, {
             'project': {
                 'key': self.project_a},
             'issuetype': {
@@ -798,6 +783,8 @@ class IssueTests(unittest.TestCase):
             'priority': {
                 'name': 'Major'}}]
         issues = self.jira.create_issues(field_list=field_list)
+        self.assertEqual(len(issues), 2)
+        self.assertIsNotNone(issues[0]['issue'], "the first issue has not been created")
         self.assertEqual(issues[0]['issue'].fields.summary,
                          'Issue created via bulk create #1')
         self.assertEqual(issues[0]['issue'].fields.description,
@@ -805,6 +792,7 @@ class IssueTests(unittest.TestCase):
         self.assertEqual(issues[0]['issue'].fields.issuetype.name, 'Bug')
         self.assertEqual(issues[0]['issue'].fields.project.key, self.project_b)
         self.assertEqual(issues[0]['issue'].fields.priority.name, 'Major')
+        self.assertIsNotNone(issues[1]['issue'], "the second issue has not been created")
         self.assertEqual(issues[1]['issue'].fields.summary,
                          'Issue created via bulk create #2')
         self.assertEqual(issues[1]['issue'].fields.description,
@@ -872,11 +860,11 @@ class IssueTests(unittest.TestCase):
     def test_create_issues_without_prefetch(self):
         field_list = [dict(project=self.project_b,
                            summary='Test issue created',
-                           description='blahery',
+                           description='some details',
                            issuetype={'name': 'Bug'}),
                       dict(project=self.project_a,
                            summary='Test issue #2',
-                           description='fooery',
+                           description='foo description',
                            issuetype={'name': 'Bug'})]
         issues = self.jira.create_issues(field_list, prefetch=False)
 
@@ -897,10 +885,10 @@ class IssueTests(unittest.TestCase):
                                        issuetype={'name': 'Bug'})
         # customfield_10022='XSS')
         issue.update(summary='Updated summary', description='Now updated',
-                     issuetype={'name': 'Improvement'})
+                     issuetype={'name': 'Story'})
         self.assertEqual(issue.fields.summary, 'Updated summary')
         self.assertEqual(issue.fields.description, 'Now updated')
-        self.assertEqual(issue.fields.issuetype.name, 'Improvement')
+        self.assertEqual(issue.fields.issuetype.name, 'Story')
         # self.assertEqual(issue.fields.customfield_10022, 'XSS')
         self.assertEqual(issue.fields.project.key, self.project_b)
         issue.delete()
@@ -914,14 +902,14 @@ class IssueTests(unittest.TestCase):
             'summary': 'Issue is updated',
             'description': "it sure is",
             'issuetype': {
-                'name': 'Improvement'},
+                'name': 'Story'},
             # 'customfield_10022': 'DOC',
             'priority': {
                 'name': 'Major'}}
         issue.update(fields=fields)
         self.assertEqual(issue.fields.summary, 'Issue is updated')
         self.assertEqual(issue.fields.description, 'it sure is')
-        self.assertEqual(issue.fields.issuetype.name, 'Improvement')
+        self.assertEqual(issue.fields.issuetype.name, 'Story')
         # self.assertEqual(issue.fields.customfield_10022, 'DOC')
         self.assertEqual(issue.fields.priority.name, 'Major')
         issue.delete()
@@ -972,9 +960,9 @@ class IssueTests(unittest.TestCase):
     @not_on_custom_jira_instance
     def test_createmeta(self):
         meta = self.jira.createmeta()
-        ztravisdeb_proj = find_by_key(meta['projects'], self.project_b)
+        proj = find_by_key(meta['projects'], self.project_b)
         # we assume that this project should allow at least one issue type
-        self.assertGreaterEqual(len(ztravisdeb_proj['issuetypes']), 1)
+        self.assertGreaterEqual(len(proj['issuetypes']), 1)
 
     @not_on_custom_jira_instance
     def test_createmeta_filter_by_projectkey_and_name(self):
@@ -986,7 +974,7 @@ class IssueTests(unittest.TestCase):
     @not_on_custom_jira_instance
     def test_createmeta_filter_by_projectkeys_and_name(self):
         meta = self.jira.createmeta(projectKeys=(self.project_a,
-                                                 self.project_b), issuetypeNames='Improvement')
+                                                 self.project_b), issuetypeNames='Story')
         self.assertEqual(len(meta['projects']), 2)
         for project in meta['projects']:
             self.assertEqual(len(project['issuetypes']), 1)
@@ -996,13 +984,31 @@ class IssueTests(unittest.TestCase):
         projects = self.jira.projects()
         proja = find_by_key_value(projects, self.project_a)
         projb = find_by_key_value(projects, self.project_b)
+        issue_type_ids = dict()
+        full_meta = self.jira.createmeta(projectIds=(proja.id, projb.id))
+        for project in full_meta['projects']:
+            for issue_t in project['issuetypes']:
+                issue_t_id = issue_t['id']
+                val = issue_type_ids.get(issue_t_id)
+                if val is None:
+                    issue_type_ids[issue_t_id] = []
+                issue_type_ids[issue_t_id].append([project['id']])
+        common_issue_ids = []
+        for key, val in issue_type_ids.items():
+            if len(val) == 2:
+                common_issue_ids.append(key)
+        self.assertNotEqual(len(common_issue_ids), 0)
+        for_lookup_common_issue_ids = common_issue_ids
+        if len(common_issue_ids) > 2:
+            for_lookup_common_issue_ids = common_issue_ids[:-1]
         meta = self.jira.createmeta(projectIds=(proja.id, projb.id),
-                                    issuetypeIds=('3', '4', '5'))
+                                    issuetypeIds=for_lookup_common_issue_ids)
         self.assertEqual(len(meta['projects']), 2)
         for project in meta['projects']:
-            self.assertEqual(len(project['issuetypes']), 3)
+            self.assertEqual(len(project['issuetypes']),
+                             len(for_lookup_common_issue_ids))
 
-    def test_createmeta_expando(self):
+    def test_createmeta_expand(self):
         # limit to SCR project so the call returns promptly
         meta = self.jira.createmeta(projectKeys=self.project_b,
                                     expand='projects.issuetypes.fields')
@@ -1063,20 +1069,23 @@ class IssueTests(unittest.TestCase):
         comment.delete()
 
     def test_editmeta(self):
+        expected_fields = {'assignee',
+                           'attachment',
+                           'comment',
+                           'components',
+                           'description',
+                           'environment',
+                           'fixVersions',
+                           'issuelinks',
+                           'labels',
+                           'summary',
+                           'versions'
+                           }
         for i in (self.issue_1, self.issue_2):
             meta = self.jira.editmeta(i)
-            self.assertTrue('assignee' in meta['fields'])
-            self.assertTrue('attachment' in meta['fields'])
-            self.assertTrue('comment' in meta['fields'])
-            self.assertTrue('components' in meta['fields'])
-            self.assertTrue('description' in meta['fields'])
-            self.assertTrue('duedate' in meta['fields'])
-            self.assertTrue('environment' in meta['fields'])
-            self.assertTrue('fixVersions' in meta['fields'])
-            self.assertTrue('issuelinks' in meta['fields'])
-            self.assertTrue('issuetype' in meta['fields'])
-            self.assertTrue('labels' in meta['fields'])
-            self.assertTrue('versions' in meta['fields'])
+            meta_field_set = set(meta['fields'].keys())
+            self.assertEqual(meta_field_set.intersection(expected_fields),
+                             expected_fields)
 
     # Nothing from remote link works
     #    def test_remote_links(self):
@@ -1146,12 +1155,12 @@ class IssueTests(unittest.TestCase):
     #                                         application={'name': 'far too silly', 'type': 'sketch'}, relationship='mousebending')
     # creation response doesn't include full remote link info, so we fetch it again using the new internal ID
     #        link = self.jira.remote_link('BULK-3', link.id)
-    #        link.update(object={'url': 'http://yahoo.com', 'title': 'yahooery'}, globalId='python-test:updated.id',
+    #        link.update(object={'url': 'http://yahoo.com', 'title': 'yahoo stuff'}, globalId='python-test:updated.id',
     #                    relationship='cheesing')
     #        self.assertEqual(link.globalId, 'python-test:updated.id')
     #        self.assertEqual(link.relationship, 'cheesing')
     #        self.assertEqual(link.object.url, 'http://yahoo.com')
-    #        self.assertEqual(link.object.title, 'yahooery')
+    #        self.assertEqual(link.object.title, 'yahoo stuff')
     #        link.delete()
     #
     #    @unittest.skip("temporary disabled")
@@ -1359,7 +1368,7 @@ class IssueLinkTests(unittest.TestCase):
                                                   JiraTestManager().project_b_issue1,
                                                   JiraTestManager().project_b_issue2)
 
-    def test_create_issue_link_with_issue_objs(self):
+    def test_create_issue_link_with_issue_obj(self):
         inwardissue = self.manager.jira_admin.issue(
             JiraTestManager().project_b_issue1)
         self.assertIsNotNone(inwardissue)
@@ -1596,6 +1605,18 @@ class SearchTests(unittest.TestCase):
         for issue in issues:
             self.assertTrue(issue.key.startswith(self.project_b))
 
+    def test_search_issues_async(self):
+        original_val = self.jira._options['async']
+        try:
+            self.jira._options['async'] = True
+            issues = self.jira.search_issues('project=%s' % self.project_b,
+                                             maxResults=False)
+            self.assertEqual(len(issues), issues.total)
+            for issue in issues:
+                self.assertTrue(issue.key.startswith(self.project_b))
+        finally:
+            self.jira._options['async'] = original_val
+
     def test_search_issues_maxresults(self):
         issues = self.jira.search_issues('project=%s' % self.project_b,
                                          maxResults=10)
@@ -1615,7 +1636,7 @@ class SearchTests(unittest.TestCase):
         self.assertFalse(hasattr(issues[0].fields, 'reporter'))
         self.assertFalse(hasattr(issues[0].fields, 'progress'))
 
-    def test_search_issues_expandos(self):
+    def test_search_issues_expand(self):
         issues = self.jira.search_issues('key=%s' % self.issue,
                                          expand='changelog')
         # self.assertTrue(hasattr(issues[0], 'names'))
@@ -1686,7 +1707,7 @@ class UserTests(unittest.TestCase):
     def test_user(self):
         user = self.jira.user(self.test_manager.CI_JIRA_ADMIN)
         self.assertEqual(user.name, self.test_manager.CI_JIRA_ADMIN)
-        self.assertRegex(user.emailAddress, '.*@example.com')
+        self.assertRegex(user.emailAddress, '^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$')
 
     @pytest.mark.xfail(reason='query returns empty list')
     def test_search_assignable_users_for_projects(self):
@@ -1900,23 +1921,20 @@ class VersionTests(unittest.TestCase):
         version.delete()
         self.assertRaises(JIRAError, self.jira.version, version.id)
 
-    # def test_version_expandos(self):
-    #     pass
-
 
 @flaky
 class OtherTests(unittest.TestCase):
 
     def test_session_invalid_login(self):
         try:
-            JIRA('https://support.atlassian.com',
+            JIRA('https://jira.atlassian.com',
                  basic_auth=("xxx", "xxx"),
                  validate=True,
                  logging=False)
         except Exception as e:
             self.assertIsInstance(e, JIRAError)
             # 20161010: jira cloud returns 500
-            assert e.status_code in (401, 500)
+            assert e.status_code in (401, 500, 403)
             str(JIRAError)  # to see that this does not raise an exception
             return
         assert False
@@ -1933,7 +1951,7 @@ class SessionTests(unittest.TestCase):
         self.assertIsNotNone(user.raw['session'])
 
     def test_session_with_no_logged_in_user_raises(self):
-        anon_jira = JIRA('https://support.atlassian.com', logging=False)
+        anon_jira = JIRA('https://jira.atlassian.com', logging=False)
         self.assertRaises(JIRAError, anon_jira.session)
 
     # @pytest.mark.skipif(platform.python_version() < '3', reason='Does not work with Python 2')
@@ -1947,6 +1965,64 @@ class SessionTests(unittest.TestCase):
         self.assertTrue(False, "Instantiation of invalid JIRA instance succeeded.")
 
 
+class AsyncTests(unittest.TestCase):
+
+    def setUp(self):
+        self.jira = JIRA('https://jira.atlassian.com', logging=False,
+                         async_=True, validate=False, get_server_info=False)
+
+    def test_fetch_pages(self):
+        """Tests that the JIRA._fetch_pages method works as expected. """
+        params = {"startAt": 0}
+        total = 26
+        expected_results = []
+        for i in range(0, total):
+            result = _create_issue_result_json(
+                i, 'summary %s' % i, key='KEY-%s' % i)
+            expected_results.append(result)
+        result_one = _create_issue_search_results_json(
+            expected_results[:10], max_results=10, total=total)
+        result_two = _create_issue_search_results_json(
+            expected_results[10:20], max_results=10, total=total)
+        result_three = _create_issue_search_results_json(
+            expected_results[20:], max_results=6, total=total)
+        mock_session = mock.Mock(name='mock_session')
+        responses = mock.Mock(name='responses')
+        responses.content = '_filler_'
+        responses.json.side_effect = [result_one, result_two, result_three]
+        responses.status_code = 200
+        mock_session.request.return_value = responses
+        mock_session.get.return_value = responses
+        self.jira._session.close()
+        self.jira._session = mock_session
+        items = self.jira._fetch_pages(Issue, 'issues', 'search', 0, False, params)
+        self.assertEqual(len(items), total)
+        self.assertEqual(
+            set(item.key for item in items),
+            set(expected_r['key'] for expected_r in expected_results))
+
+
+def _create_issue_result_json(issue_id, summary, key, **kwargs):
+    """Returns a minimal json object for an issue. """
+    return {
+        'id': '%s' % issue_id,
+        'summary': summary,
+        'key': key,
+        'self': kwargs.get('self', 'http://example.com/%s' %
+                           issue_id),
+    }
+
+
+def _create_issue_search_results_json(issues, **kwargs):
+    """Returns a minimal json object for Jira issue search results. """
+    return {
+        'startAt': kwargs.get('start_at', 0),
+        'maxResults': kwargs.get('max_results', 50),
+        'total': kwargs.get('total', len(issues)),
+        'issues': issues,
+    }
+
+
 @flaky
 class WebsudoTests(unittest.TestCase):
 
@@ -1971,8 +2047,19 @@ class UserAdministrationTests(unittest.TestCase):
         self.test_password = rndpassword()
         self.test_groupname = 'testGroupFor_%s' % self.test_manager.project_a
 
-    def test_add_and_remove_user(self):
+    def _skip_pycontribs_instance(self):
+        pytest.skip('The current ci jira admin user for '
+                    'https://pycontribs.atlassian.net lacks '
+                    'permission to modify users.')
 
+    def _should_skip_for_pycontribs_instance(self):
+        return self.test_manager.CI_JIRA_ADMIN == 'ci-admin' and (
+            self.test_manager.CI_JIRA_URL ==
+            "https://pycontribs.atlassian.net")
+
+    def test_add_and_remove_user(self):
+        if self._should_skip_for_pycontribs_instance():
+            self._skip_pycontribs_instance()
         try:
             self.jira.delete_user(self.test_username)
         except JIRAError:
@@ -2004,8 +2091,19 @@ class UserAdministrationTests(unittest.TestCase):
         self.assertEqual(
             len(x), 0, "Found test user when it should have been deleted. Test Fails.")
 
+        # test creating users with no application access (used for Service Desk)
+        result = self.jira.add_user(
+            self.test_username, self.test_email, password=self.test_password,
+            application_keys=['jira-software'])
+        assert result, True
+
+        result = self.jira.delete_user(self.test_username)
+        assert result, True
+
     @flaky
     def test_add_group(self):
+        if self._should_skip_for_pycontribs_instance():
+            self._skip_pycontribs_instance()
         try:
             self.jira.remove_group(self.test_groupname)
         except JIRAError:
@@ -2021,6 +2119,8 @@ class UserAdministrationTests(unittest.TestCase):
         self.jira.remove_group(self.test_groupname)
 
     def test_remove_group(self):
+        if self._should_skip_for_pycontribs_instance():
+            self._skip_pycontribs_instance()
         try:
             self.jira.add_group(self.test_groupname)
             sleep(1)  # avoid 400: https://travis-ci.org/pycontribs/jira/jobs/176539521#L395
@@ -2067,6 +2167,8 @@ class UserAdministrationTests(unittest.TestCase):
         self.jira.delete_user(self.test_username)
 
     def test_remove_user_from_group(self):
+        if self._should_skip_for_pycontribs_instance():
+            self._skip_pycontribs_instance()
         try:
             self.jira.add_user(
                 self.test_username, self.test_email, password=self.test_password)
@@ -2104,6 +2206,36 @@ class JiraShellTests(unittest.TestCase):
         self.assertEqual(result, 0)
 
 
+class JiraServiceDeskTests(unittest.TestCase):
+
+    def setUp(self):
+        self.jira = JiraTestManager().jira_admin
+        self.test_manager = JiraTestManager()
+
+    def test_create_customer_request(self):
+        if not self.jira.supports_service_desk():
+            pytest.skip('Skipping Service Desk not enabled')
+
+        try:
+            self.jira.create_project('TESTSD', template_name='IT Service Desk')
+        except JIRAError:
+            pass
+        service_desk = self.jira.service_desks()[0]
+        request_type = self.jira.request_types(service_desk)[0]
+
+        request = self.jira.create_customer_request(dict(
+            serviceDeskId=service_desk.id,
+            requestTypeId=int(request_type.id),
+            requestFieldValues=dict(
+                summary='Ticket title here',
+                description='Ticket body here'
+            )
+        ))
+
+        self.assertEqual(request.fields.summary, 'Ticket title here')
+        self.assertEqual(request.fields.description, 'Ticket body here')
+
+
 if __name__ == '__main__':
 
     # when running tests we expect various errors and we don't want to display them by default
